function value = get_value( this, op_name, var_name, ind, mode )

% value = get_value( this, op_name, var_name, ind, mode )
%
% Returns a single value or row from the results. If var_name is [], then
% the whole row is returned. Mode can be either 'iters' or 'calls':
%
%   'iters'   results are viewed at iteration level. 'ind' is interpreted as
%             the result row number.
%
%   'calls'   results are viewed at call level. 'ind' is interpreted as the
%             call number.
%
%
% 'calls' mode:
%
% Initial value of the variable is substracted from the final value, and the
% result is returned. If var_name is [], then all variables are processed
% and returned as a vector.
%
% Efficiency variable is an exception: it is corrected to reflect the
% average efficiency over the whole call.
%
% Also some extra variables are added: 'call iters' contains the
% iteration count inside the call, 'cputimer start' and 'cputimer stop'
% contain the cputimer readings when the call started and ended. The
% internal 'start iteration' is compensated away from 'call iters', so this
% value contains the real iteration count. The extra variables are always
% added after the original ones in same order as above. Also get_var_ind()
% recognizes these special variables.


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.



% The indices of variables cputimer, total cost and efficiency are
% hardcoded here!
%
% Some of the 'calls' functionality is duplicated in get_res, remember to
% modify it along with this.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


op_ind = get_op_ind( this, op_name );


switch mode
  
 case 'iters'
  
  if isempty( var_name )
    value = this.results{ op_ind }( ind, : );
  else
    value = this.results{ op_ind } ...
            ( ind, get_var_ind(this, op_name, var_name) );
  end
  
  
 case 'calls'
  
  % get the first and last rows
  frow_ind  = this.areas{op_ind}(ind);
  lrow_ind  = this.areas{op_ind}(ind + 1) - 1;
  first_row = get_value( this, op_name, [], frow_ind, 'iters' );
  last_row  = get_value( this, op_name, [], lrow_ind, 'iters' );
  
  % substract first row from last and add the extra variables
  % ('start iteration' is compensated in iteration count!)
  last_row  = [ (last_row - first_row), lrow_ind - frow_ind, ...
                first_row(1), last_row(1) ];
  
  % correct the efficiency
  if last_row(1) == 0
    if last_row(2) == 0, last_row(3) = 0; else last_row(3) = NaN; end
  else
    last_row(3) = - last_row(2) / last_row(1);
  end
  
  % separate the requested variable if needed
  if isempty(var_name)
    value = last_row;
  else
    value = last_row( get_var_ind(this, op_name, var_name) );
  end
  
end


return;
