function cs = cs_adaptive( data, state, varargin )

% cs = cs_adaptive( data, state, [cs_args] )
%
% Default Settings for ctrl_adaptive. Accepted extra arguments are explained in
% the help section of ivga().
%
% Operation probabilities:
%   - All operation probabilities are adapted during evaluation.
%   - Split is enabled when group_count <= variable_count / 2
%     (i.e., average group size >= 2).
%   - Split is always disabled if already splitting and all variables are in
%     one group (this is hard-coded into Split()).
%
% Default stopping criterion:
%   The 'normal' preset, see the help section of ivga().
%
% Split:
%   - Maximum split recursion depth is limited to 1, except when splitting a
%     single group which contains all variables: in this case a 'free' split
%     recursion level is given, i.e. the maximum recursion depth limit is not
%     decremented when making the call.
%   - Stopping criterion:
%       Evaluation is stopped if either the group count drops to 1, or 3 *
%       variable_count (local variable count) iterations have elapsed.


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% select the modeler and controller
cs.mdl.class  = 'mdl_vq_real_nominal';
cs.ctrl.class = 'ctrl_adaptive';




% ---- standard initialization ----


% parse the data dimensions and group setup
[n, dim, setup_mode, setup_params] = ...
    cshelper_parse_grouping_and_dim( cs, data, state, varargin{:} );

% store dimensionality for internal use
cs.private.n = n; cs.private.dim = dim;




% ---- search strategies ----


cs.stl = state_transition_limiter( ...
    'neighborhood_limiter', null_limiter, ...
    'gradient_limiter', greedy_search );




% ---- modeler ----


cs.mdl.max_points       = round(n/2);
cs.mdl.ncentroids_def   = 2;
cs.mdl.iters            = 30;
cs.mdl.check            = 10;




% ---- controller ----


% -- init_groups --

% group setup
cs.ctrl.init_groups.setup_mode    = setup_mode;
cs.ctrl.init_groups.setup_params  = setup_params;

% group model computation
cs.ctrl.init_groups.Reinit_model.stop_trigger    = ...
    trigger_iters( 10, flt_stats_reader('init_groups - Reinit_model','iters'));
cs.ctrl.init_groups.Improve_model.stop_trigger   = ...
    trigger_iters( 0, flt_stats_reader('init_groups - Improve_model','iters'));


% -- tune_groups --

% stopping
if ~isarg('runtime'), varargin = { varargin{:}, 'runtime', 'normal' }; end
cs.ctrl.tune_groups.stop_trigger        = NaN;   % placeholder
cs.ctrl.tune_groups.sweeps_to_stabilize = Inf;   % disabled

% split enabling
cs.ctrl.tune_groups.max_split_depth   = 1;
cs.ctrl.tune_groups.split_max_ngroups = dim / 2;

% primitive operation probability adaptation
cs.ctrl.tune_groups.op_pr.lead_in            = 10;
cs.ctrl.tune_groups.op_pr.lead_in_pr         = [0.3 0.5 1.0 0.25 0.1];
cs.ctrl.tune_groups.op_pr.min_pr             = [0.1 0.1 0.1 0.1 0.05];
cs.ctrl.tune_groups.op_pr.cputimer_lowpass   = 0.9876;
cs.ctrl.tune_groups.op_pr.efficiency_lowpass = 0.9980;

% misc
cs.ctrl.tune_groups.memcheck_interval = 100;




% ---- primitive operations ----


% Reinit_model
%cs.primitive_ops.Reinit_model.stop_trigger = ...
cs.ctrl.tune_groups.Reinit_model.stop_trigger = ...
    trigger_iters( 1, flt_stats_reader('Reinit_model', 'iters') );
% Improve_model
cs.ctrl.tune_groups.Improve_model.stop_trigger = ...
    trigger_iters( 1, flt_stats_reader('Improve_model', 'iters') );
% Move_var
cs.ctrl.tune_groups.Move_var.reinit_lim_trigger  = ...
    trigger_iters( 10, flt_stats_reader('Move_var - Reinit_model', 'iters') );
cs.ctrl.tune_groups.Move_var.improve_lim_trigger = ...
    trigger_iters( 0, flt_stats_reader('Move_var - Improve_model', 'iters') );
% Merge
cs.ctrl.tune_groups.Merge.Reinit_model.stop_trigger  = ...
    trigger_iters( 1, flt_stats_reader('Merge - Reinit_model', 'iters') );
cs.ctrl.tune_groups.Merge.Improve_model.stop_trigger = ...
    trigger_iters( 0, flt_stats_reader('Merge - Improve_model', 'iters') );
% Split
cs.ctrl.tune_groups.Split.cs                    = @cs_adaptive_split;
cs.ctrl.tune_groups.Split.current_split_depth   = 0;




% ---- standard finalization ----


% apply common extra arguments
cs = cshelper_apply_common_exargs( cs, data, state, varargin{:} );


return;








function cs = cs_adaptive_split( data, state, varargin )

% Settings for the recursive Split call.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% inherit from cs_adaptive
cs  = cs_adaptive( data, state, varargin{:} );
dim = cs.private.dim;


% -- override some fields (shorten the runtime) --

% init_groups
cs.ctrl.init_groups.Reinit_model.stop_trigger = ...
    trigger_iters(1, flt_stats_reader('init_groups - Reinit_model','iters'));
cs.ctrl.init_groups.Improve_model.stop_trigger = ...
    trigger_iters(1, flt_stats_reader('init_groups - Improve_model','iters'));

% tune_groups - stopping
cs.ctrl.tune_groups.stop_trigger = trigger_or( ...
    trigger_range( ...   % stop immediately if all vars go into one group
        [1 1 1 dim], ...
        flt_stats_reader({'tune_groups', 'group count'}, 'iters')), ...
    trigger_iters( 3 * dim, flt_stats_reader('tune_groups', 'iters') ));
cs.ctrl.tune_groups.sweeps_to_stabilize = Inf;


return;
