function res = ivga_show_stats( source, varargin )

% IVGA_SHOW_STATS Show learning statistics about an IVGA result or results.
%
%   data = ivga_show_stats( source, [options ...] )
%
%   data = ivga_show_stats( results )
%   data = ivga_show_stats( results, 'y_var', 'group count' )
%   data = ivga_show_stats( {results1, results2} )
%
% Arguments
%
%   source      The results to be analyzed. This can be a single result
%               struct or a batch result array from ivga(), a filename or the
%               id string of a batch saved with the 'save_as' option, or a
%               combination of these combined with the {} operators, e.g.,
%               {result_data1, {result_data2_run1, result_data2_run2}}
%
% Options
%
%   Selecting the data to be plotted:
%
%     ['op', op],
%     [ ['x_var', x_var], ['y_var', y_var] | ['final_value', var] ]
%
%   Controlling averaging:
%
%     ['dont_average'],
%     ['separate', 'entries' | 'datasets' | 'repetitions' | 'none' | dim],
%     ['mode', 'iters' | 'calls']
%
%   Plotting:
%
%     ['stddev', '' | 'x' | 'y' | 'xy'],
%     ['color', color],
%     ['dont_clear'], ['nodisplay']
%
% Available variables for the 'x_var', 'y_var' and 'final_value' options:
%   cputimer (default for x_var), total cost (default for y_var), efficiency,
%   group count, stop trigger level, stationary variable sweeps, iteration
%
% The following are also available when using the adaptive mode (with
% @cs_adaptive):
%   memory usage, pr_Reinit_model, pr_Improve_model, pr_Move_var, pr_Merge,
%   pr_Split, share_Reinit_model, share_Improve_model, share_Move_var,
%   share_Merge, share_Split
%
%
% Examples
%
%   D = rand(250,2); D = [D D];
%   results1 = ivga( D )
%   results2 = ivga( D, 'runs', 3 )
%   ivga( D, @cs_adaptive, 'runs', 3, 'save_as', 'test' )
%
%   Basic usage:
%     ivga_show_stats( results1 )
%     ivga_show_stats( results2 )
%     ivga_show_stats( 'test' )
%     ivga_show_stats( './batch_test_results.mat' )
%
%   Selecting the data to be plotted:
%     ivga_show_stats( 'test', 'y_var', 'iteration' )
%     ivga_show_stats( 'test', 'x_var', 'iteration', 'y_var', 'group count' )
%     ivga_show_stats( 'test', 'final_value', 'group count' )
%
%   Combining multiple results:
%     ivga_show_stats( {results1, results2, 'test'} )
%     ivga_show_stats( {results1, results2, 'test'}, ...
%                                 'final_value', 'group count' )
%
%   Displaying all repetitions separately:
%     ivga_show_stats( {results1, results2}, 'dont_average' )
%     ivga_show_stats( 'test', 'separate', 'repetitions' )
%     ivga_show_stats( 'test', 'separate', 'repetitions', ...
%                              'y_var', 'group count' )
%
%   Selecting the standard deviation plotting style:
%     ivga_show_stats( 'test', 'stddev', '' )
%     ivga_show_stats( 'test', 'stddev', 'x' )
%     ivga_show_stats( 'test', 'stddev', 'y' )
%     ivga_show_stats( 'test', 'stddev', 'xy' )
%
% See also IVGA, IVGA_SHOW


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.



% Implementation details
% ----------------------
%
% The actual processing and averaging of the results is delegated to the
% average_results() function (in average_results.m), which returns a struct
% array and some other variables containing the final data that the user
% requested: most of the arguments are actually passed directly to
% average_results() and processed there. This function then only takes care of
% the visualization of the received data.
%
% The data from average_results() is always returned to the user at the end,
% and by default also plotted somehow. Two types of plots exist:
%   - 2-dimensional plots of a variable as a function of another variable.
%     (handled by the plot_xy_entry() helper function)
%   - Bar plots from the final values of a selected variable.
%     (handled by the plot_fv_entry() helper function)
% Each element from the result struct array will convert to a single curve or a
% single bar in the plot. These will be iterated in the big for loop in the
% main function body and each element is plotted with the appropriate helper
% function mentioned above.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% constants
colors      = 'brkmgc';   % or: 'bgrcmyk'
line_widths = [0.5 2:4];




% ---- check arguments ----


% fill in defaults
stddev     = 'xy';
color      = colors(1);

% overwrite given arguments
[flg,value] = isarg('stddev'); if flg, stddev = value; end
[flg,value] = isarg('color'); if flg, color = value; end

% resolve boolean arguments
dont_clear = isarg('dont_clear');
nodisplay  = isarg('nodisplay');




% ---- compute and plot the curve(s) ----


% compute the averaged curve(s)
[res, op, x_var, y_var, fv_var] = average_results( source, varargin{:} );
if isempty( fv_var ), mode = 'xy'; else mode = 'fv'; end
names = {res.name};

% plot the curve(s) and remove empty entries
if ~nodisplay
  if ~dont_clear, clf; end
  hold on;
end
ind = 1;
for i=1:length(res)
  
  % select color
  if length(res) == 1
    current_color = color;
  else
    current_color = colors(mod(ind-1,length(colors))+1);
    if i == 1 && ~nodisplay && isarg('color')
      warning([ '''color'' argument ignored because of multiple ' ...
                'result entries.' ]);
    end
  end
  
  % plot
  switch mode
   case 'xy'
    if isempty( res(i).X )
      fprintf('Note: entry %d is empty and is omitted from the results.\n', i);
      handles(i) = NaN;
    else
      if nodisplay, handles(i) = 0; else
        handles(i) = ...
            plot_xy_entry( res(i), current_color, ...
                           line_widths(mod(floor((ind-1)/length(colors)), ...
                                           length(line_widths))+1), stddev );
      end
      ind = ind + 1;
    end
   case 'fv'
    if isempty( res(i).value )
      fprintf('Note: entry %d is empty and is omitted from the results.\n', i);
      handles(i) = NaN;
    else
      if nodisplay, handles(i) = 0; else
        handles(i) = plot_fv_entry( res(i), current_color, stddev, ind );
      end
      names{i} = cat_fv_values( names{i}, res(i), ind );
      ind = ind + 1;
    end
   otherwise
    error('Internal error!');
  end
  
end

% discard names and handles of empty entries
names(isnan(handles))   = [];
handles(isnan(handles)) = [];

% add legend and labels
if ~nodisplay
  if strcmp( mode, 'fv' )
    legend( handles, escstr(names), 'Location', 'NorthEastOutside' );
    ylabel( ['final value of: ' escstr(fv_var)] );
    xlim([ 0.5, ind - 0.5 ]);
    
    if any(stddev == 'y')
      y_minmax = [ min([res.value] - [res.stddev]), ...
                max([res.value] + [res.stddev]) ];
      ylim( mean(y_minmax) + ...
            1.2 * (y_minmax - mean(y_minmax)) );
    else
      ylim( mean([res.value]) + ...
            1.2 * (minmax([res.value]) - mean([res.value])) );
    end
    grid on;
  else
    legend( handles, escstr(names) );
    xlabel( escstr(x_var) ); ylabel( escstr(y_var) );
  end
  if ~strcmp( op, 'tune_groups' ) || isarg('op')
    title([ 'operation: ' escstr(op) ]);
  end
end




% ---- wrap up and return the result ----


% ...

return;








% ---- plot_xy_entry() and helper functions begin ----




function handle = plot_xy_entry( res, color, width, stddev )

% Plots a single entry.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% no stddev plots if only one result
if length(res.all) == 1, stddev = ''; end

% compute various X indices and receive the possibly expanded average curve
[res_extrap, X_inds] = compute_X_inds( res );

% plot
handle = plot_main_curve( res_extrap, X_inds, color, width, ...
                          any(stddev == 'x') );
plot_stddev_curves( res_extrap, X_inds, color, width, any(stddev == 'y' ) );

return;




function [res, X_inds] = compute_X_inds( res )

% The indices are computed for the full length average vectors in res. (which
% will be extrapolated if needed for the x-stddev inds!)
%
% Fields in X_inds struct:
%   .Xmin_stddev_start
%   .Xmin_mean
%   .Xmin_stddev_end
%   .Xmax_stddev_start
%   .Xmax_mean
%   .Xmax_stddev_end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% positions on X to find inds for
positions = [ res.minmax_stats.Xmin_mean - res.minmax_stats.Xmin_stddev, ...
              res.minmax_stats.Xmin_mean, ...
              res.minmax_stats.Xmin_mean + res.minmax_stats.Xmin_stddev, ...
              res.minmax_stats.Xmax_mean - res.minmax_stats.Xmax_stddev, ...
              res.minmax_stats.Xmax_mean, ...
              res.minmax_stats.Xmax_mean + res.minmax_stats.Xmax_stddev ];

% extrapolate the X, Y and stddev vectors if needed
if min(positions) < res.X(1)
  res.X      = [ min(positions), res.X ];
  res.Y      = [ res.Y(1), res.Y ];
  res.stddev = [ res.stddev(1), res.stddev ];
end
if max(positions) > res.X(end)
  res.X      = [ res.X, max(positions) ];
  res.Y      = [ res.Y, res.Y(end) ];
  res.stddev = [ res.stddev, res.stddev(end) ];
end

% find the inds for the positions
[dummy, inds] = min(abs( repmat(res.X', 1, length(positions)) - ...
                         repmat(positions, length(res.X), 1) ));
X_inds.Xmin_stddev_start = inds(1);
X_inds.Xmin_mean         = inds(2);
X_inds.Xmin_stddev_end   = inds(3);
X_inds.Xmax_stddev_start = inds(4);
X_inds.Xmax_mean         = inds(5);
X_inds.Xmax_stddev_end   = inds(6);

% find the inds for all start- and endpoints
%[dummy, X_inds.all.Xmins] = ...
%    min(abs( repmat(res.X', 1, length([res.all.Xmin])) - ...
%             repmat([res.all.Xmin], length(res.X), 1) ));
%[dummy, X_inds.all.Xmaxs] = ...
%    min(abs( repmat(res.X', 1, length([res.all.Xmax])) - ...
%             repmat([res.all.Xmax], length(res.X), 1) ));

return;




function handle = plot_main_curve( res, X_inds, color, width, x_stddev )

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if x_stddev
  
  % dotted x-stddev plot at the beginning
  range = X_inds.Xmin_stddev_start : X_inds.Xmin_mean;
  plot( res.X(range), res.Y(range), [color ':'], 'LineWidth', width );
  
  % the x-stddev x-marks at the beginning
  plot( res.X(X_inds.Xmin_stddev_start), ...
        res.Y(X_inds.Xmin_stddev_start), [color '+'] );
  plot( res.X(X_inds.Xmin_stddev_end), ...
        res.Y(X_inds.Xmin_stddev_end), [color '+'] );
  
  % the x-stddev x-marks at the end
  plot( res.X(X_inds.Xmax_stddev_start), ...
        res.Y(X_inds.Xmax_stddev_start), [color '+'] );
  plot( res.X(X_inds.Xmax_stddev_end), ...
        res.Y(X_inds.Xmax_stddev_end), [color '+'] );
  
  % dotted x-stddev plot at the end
  range = X_inds.Xmax_mean : X_inds.Xmax_stddev_end;
  plot( res.X(range), res.Y(range), [color ':'], 'LineWidth', width );
  
end

% the main average-length solid plot
range = X_inds.Xmin_mean:X_inds.Xmax_mean;
handle = plot( res.X(range), res.Y(range), color, 'LineWidth', width );

% all X start- and endpoints of the original curves
%plot( res.X(X_inds.all.Xmins), res.Y(X_inds.all.Xmins), [color '>'], ...
%      'MarkerSize', 4 );
%plot( res.X(X_inds.all.Xmaxs), res.Y(X_inds.all.Xmaxs), [color '<'], ...
%      'MarkerSize', 4 );

return;




function plot_stddev_curves( res, X_inds, color, width, y_stddev )

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if y_stddev
  
  % the dotted y-stddev curves on top and below of the main solid curve
  range = X_inds.Xmin_mean:X_inds.Xmax_mean;
  plot( res.X(range), res.Y(range) - res.stddev(range), ...
        [color ':'], 'LineWidth', width );
  plot( res.X(range), res.Y(range) + res.stddev(range), ...
        [color ':'], 'LineWidth', width );
  
end

return;




% ---- plot_xy_entry() and helper functions end ----








% ---- plot_fv_entry() and helper functions begin ----




function handle = plot_fv_entry( res, color, stddev, ind )

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


handle = bar( ind, res.value, color );
if any(stddev == 'y')
  plot( [ind ind], [res.value - res.stddev, res.value + res.stddev], 'k-+' );
end

return;




function str = cat_fv_values( str, res, ind )

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


str = [ num2str(ind) '. ' str '\newline' ...
        '    final value: ' num2str(res.value) '\newline' ...
        '    standard deviation: ' num2str(res.stddev) '\newline' ];

return;




% ---- plot_fv_entry() and helper functions end ----








function str = escstr( str )

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


str = strrep( str, '_', '\_' );

return;
